/*
 *	split.c
 *
 *	Program to split a binary file into specified size pieces
 *	V1.01, Public Domain (p) 1993, S. R. Sampson, N5OWK
 *	No rights reserved
 *
 *	I made this because I accidently combined a bunch of files
 *	and then had to break them apart again.  I tried the R95.EXE
 *	utility, but it couldn't handle big files (13 Meg).
 *
 *	The algorithm used, is to split an input file into the Operating
 *	Systems BUFSIZ blocks.  The blocks are then copied from the
 *	source to the destination.  The result was acceptable in speed.
 *
 *	Borland C++ 2.0
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static FILE	*Fpi = (FILE *)NULL;
static FILE	*Fpo = (FILE *)NULL;

static long	Split_Size = 0L;
static long	Last_Piece = 0L;
static long	File_Size = 0L;

static int	Chunks = 0;


void main(int argc, char **argv)
{
	register int	i, j;
	int		buffer_chunks;
	int		buffer_extra;
	char		buffer[BUFSIZ];
	char		outname[16];

	if (argc != 4)  {
		fprintf(stderr, "Usage: split <file> <split-size> <prefix>\n");
		fprintf(stderr, "Where the prefix name is appended with .000");
		fprintf(stderr, " through .999\n");
		exit(1);
	}

	Split_Size = atol(argv[2]);	/* no error checking */

	if ((Fpi = fopen(argv[1], "rb")) == NULL)  {
		fprintf(stderr, "Could not locate requested file\n");
		exit(1);
	}

	fseek(Fpi, 0L, SEEK_END);
	File_Size = ftell(Fpi);
	fseek(Fpi, 0L, SEEK_SET);

	if (Split_Size >= File_Size)  {
		fprintf(stderr, "Source file is smaller than split size.\n");
		exit(1);
	}

	Chunks = File_Size / Split_Size;
	Last_Piece = File_Size % Split_Size;

	if ((Chunks + (Last_Piece != 0L)) > 999)  {
		fprintf(stderr, "Too many files to produce (> 999)\n");
		exit(1);
	}

	printf("\nFile size: %ld\nSplit size: %ld\nNumber of files: %d\n\n",
		File_Size, Split_Size, Chunks);

	for (i = 0; i < Chunks; i++)  {
		sprintf(outname, "%s.%03d", argv[3], i);
		printf("-> %s\r", outname);

		if ((Fpo = fopen(outname, "wb")) == NULL)  {
			fprintf(stderr, "\nWrite error. Check file space\n");
			exit(1);
		}

		buffer_chunks = Split_Size / BUFSIZ;
		buffer_extra = Split_Size % BUFSIZ;

		for (j = 0; j < buffer_chunks; j++)  {
			fread(buffer, BUFSIZ, 1, Fpi);
			fwrite(buffer, BUFSIZ, 1, Fpo);
		}

		if (buffer_extra)  {
			fread(buffer, buffer_extra, 1, Fpi);
			fwrite(buffer, buffer_extra, 1, Fpo);
		}

		fclose(Fpo);
	}

	if (Last_Piece)  {
		sprintf(outname, "%s.%03d", argv[3], Chunks);
		printf("-> %s\r", outname);

		if ((Fpo = fopen(outname, "wb")) == NULL)  {
			fprintf(stderr, "\nWrite error. Check file space\n");
			exit(1);
		}

		buffer_chunks = Last_Piece / BUFSIZ;
		buffer_extra = Last_Piece % BUFSIZ;

		for (j = 0; j < buffer_chunks; j++)  {
			fread(buffer, BUFSIZ, 1, Fpi);
			fwrite(buffer, BUFSIZ, 1, Fpo);
		}

		if (buffer_extra)  {
			fread(buffer, buffer_extra, 1, Fpi);
			fwrite(buffer, buffer_extra, 1, Fpo);
		}
	}

	printf("                                        \n");

	fclose(Fpo);
	fclose(Fpi);

	exit(0);
}
